package cn.fw.valhalla.domain.enums;

import com.baomidou.mybatisplus.core.enums.IEnum;
import com.fasterxml.jackson.annotation.JsonCreator;
import com.fasterxml.jackson.annotation.JsonValue;
import lombok.Getter;

/**
 * @author : kurisu
 * @className : ReachInvalidReasonEnum
 * @description : 进站数据无效原因
 * @date: 2023-08-16 17:37
 */
public enum ReachInvalidReasonEnum implements IEnum<Integer> {
    /**
     * 不适用
     */
    INAPPLICABLE(1, "不适用"),
    /**
     * 自然进站
     */
    NATURAL(2, "自然进站"),
    /**
     * 重复进站
     */
    DUPLICATE(3, "重复进站"),
    ;

    /**
     * 值
     */
    private final Integer value;
    /**
     * 名称
     */
    @Getter
    private final String name;

    ReachInvalidReasonEnum(final Integer value, final String name) {
        this.value = value;
        this.name = name;
    }

    /**
     * 根据枚举值获取枚举对象
     */
    @JsonCreator
    public static ReachInvalidReasonEnum ofValue(final Integer value) {
        for (final ReachInvalidReasonEnum typeEnum : ReachInvalidReasonEnum.values()) {
            if (typeEnum.value.equals(value)) {
                return typeEnum;
            }
        }
        return null;
    }

    /**
     * 获取描述
     *
     * @return 值
     */
    @JsonCreator
    public static String getNameByVale(final Integer value) {
        for (final ReachInvalidReasonEnum typeEnum : ReachInvalidReasonEnum.values()) {
            if (typeEnum.value.equals(value)) {
                return typeEnum.getName();
            }
        }
        return "";
    }

    /**
     * 获取值
     *
     * @return 值
     */
    @JsonValue
    @Override
    public Integer getValue() {
        return value;
    }
}
