package cn.fw.valhalla.sdk.param;

import lombok.Data;

import javax.validation.constraints.NotNull;
import java.util.Date;
import java.util.Objects;

/**
 * @author : kurisu
 * @className : ReachLogReq
 * @description : 进站记录
 * @date: 2020-11-11 15:35
 * @since : 1.3.2
 */
@Data
public class ReachLogReq {
    /**
     * 档案id
     */
    @NotNull(message = "档案id不能为空")
    private Long customerId;
    /**
     * 工单号
     *
     * @since : 1.3.2
     */
    private String orderNo;
    /**
     * 进站时间
     */
    @NotNull(message = "进站时间不能为空")
    private Date arrivalTime;
    /**
     * 行驶里程
     * 已过时 现在区分燃油里程和纯电里程。总里程自动计算
     *
     * @deprecated 已过时 使用新属性{@link cn.fw.valhalla.sdk.param.ReachLogReq.fuelMileage}
     */
    private Integer arrivalMileage;
    /**
     * 燃油里程
     *
     * @since : 1.3.3
     */
    private Integer fuelMileage;
    /**
     * 纯电里程
     *
     * @since : 1.3.3
     */
    private Integer electricMileage;
    /**
     * 服务站id
     */
    @NotNull(message = "服务站不能为空")
    private Long shopId;
    /**
     * 服务顾问id
     *
     * @since : 1.3.2
     */
    @NotNull(message = "服务顾问id不能为空")
    private Long adviserId;
    /**
     * 集团id
     */
    @NotNull(message = "集团id不能为空")
    private Long groupId;


    public boolean checkMileage() {
        return Objects.nonNull(arrivalMileage) || Objects.nonNull(fuelMileage) || Objects.nonNull(electricMileage);
    }
}
