package cn.fw.valhalla.controller.app

import cn.fw.common.page.AppPage
import cn.fw.common.web.annotation.ControllerMethod
import cn.fw.common.web.util.ResultBuilder.success
import cn.fw.data.base.domain.common.Message
import cn.fw.security.auth.client.annotation.Authorization
import cn.fw.security.auth.client.annotation.IgnoreAuth
import cn.fw.security.auth.client.enums.AuthType
import cn.fw.valhalla.domain.query.InitClueQueryVO
import cn.fw.valhalla.domain.vo.ipt.CustomerInitClueVO
import cn.fw.valhalla.domain.vo.ipt.InitClueCallLogVO
import cn.fw.valhalla.service.bus.ipt.CustomerInitClueBizService
import org.springframework.validation.annotation.Validated
import org.springframework.web.bind.annotation.GetMapping
import org.springframework.web.bind.annotation.RequestMapping
import org.springframework.web.bind.annotation.RestController

/**
 * 事故车处理控制器
 *
 * @author : kurisu
 * @version : 1.0
 * @desc : 事故车处理控制器
 * @date : 2024-02-29 17:50
 */
@RestController
@Authorization(AuthType.APP)
@Validated
@RequestMapping("/app/init/customer")
class InitClueController(private val customerInitClueBizService: CustomerInitClueBizService) {
    @IgnoreAuth
    @GetMapping("/pool")
    @ControllerMethod("查询档案期初数据")
    fun customerPool(queryVO: InitClueQueryVO): Message<AppPage<CustomerInitClueVO>> {
        return success(customerInitClueBizService.listPage(queryVO))
    }

    @IgnoreAuth
    @GetMapping("/history")
    @ControllerMethod("查询历史跟进数据")
    fun customerPoolHistory(clueId: Long): Message<List<InitClueCallLogVO>> {
        return success(customerInitClueBizService.callLog(clueId))
    }
}