package cn.fw.valhalla.controller.app;

import cn.fw.common.page.AppPage;
import cn.fw.common.web.annotation.ControllerMethod;
import cn.fw.common.web.auth.LoginAuthBean;
import cn.fw.common.web.auth.annotation.CurrentUser;
import cn.fw.data.base.domain.common.Message;
import cn.fw.security.auth.client.annotation.Authorization;
import cn.fw.security.auth.client.enums.AuthType;
import cn.fw.valhalla.component.producer.CustAfterDistProducer;
import cn.fw.valhalla.domain.dto.DistributableDTO;
import cn.fw.valhalla.domain.dto.LeaveAllocationDTO;
import cn.fw.valhalla.domain.enums.LeaveTodoTypeEnum;
import cn.fw.valhalla.domain.query.LeaveQueryVO;
import cn.fw.valhalla.domain.vo.DistributableVO;
import cn.fw.valhalla.domain.vo.LeaveNeedDoVO;
import cn.fw.valhalla.service.bus.LeaveNeedDoBizService;
import lombok.extern.slf4j.Slf4j;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.validation.annotation.Validated;
import org.springframework.web.bind.annotation.*;

import javax.validation.Valid;
import javax.validation.constraints.NotNull;
import java.util.Collections;
import java.util.List;

import static cn.fw.common.web.util.ResultBuilder.success;

/**
 * @author : kurisu
 * @className : LeaveNeedDoController
 * @description : 人员变动待处理事件
 * @date: 2020-08-15 14:52
 */
@Slf4j
@RestController
@Authorization(AuthType.APP)
@Validated
@RequestMapping("/app/leave2do")
public class LeaveNeedDoController {

    private final LeaveNeedDoBizService leaveNeedDoBizService;
    private final CustAfterDistProducer custAfterDistProducer;

    @Autowired
    public LeaveNeedDoController(final LeaveNeedDoBizService leaveNeedDoBizService,
                                 final CustAfterDistProducer custAfterDistProducer) {
        this.leaveNeedDoBizService = leaveNeedDoBizService;
        this.custAfterDistProducer = custAfterDistProducer;
    }

    @GetMapping("/customer/list")
    @ControllerMethod("保有客档案待分配人员列表")
    public Message<AppPage<LeaveNeedDoVO>> save(@CurrentUser LoginAuthBean currentUser, final LeaveQueryVO queryVO) {
        queryVO.setType(LeaveTodoTypeEnum.CUSTOMER.getValue());
        return success(leaveNeedDoBizService.getList(currentUser, queryVO));
    }

    @GetMapping("/customer/allocation_count")
    @ControllerMethod("查询保有客档案待分配数量")
    public Message<Long> allocationCount(@NotNull(message = "分配记录id不能为空") final Long id) {
        long l = leaveNeedDoBizService.querytoBeAssignedCount(id);
        return success(l);
    }

    @PostMapping("/customer/allocation")
    @ControllerMethod("保有客档案分配")
    public Message<Void> allocation(@CurrentUser LoginAuthBean currentUser, @RequestBody @Valid LeaveAllocationDTO dto) {
        leaveNeedDoBizService.allocation(currentUser, dto);
        custAfterDistProducer.send(Boolean.TRUE, Collections.singletonList(dto.getAdviserId()));
        return success();
    }

    @GetMapping("/customer/distributable")
    @ControllerMethod("查询门店可分配顾问列表")
    public Message<List<DistributableVO>> distributableList(@NotNull(message = "门店id不能为空") final Long shopId) {
        return success(leaveNeedDoBizService.distributableList(shopId));
    }

    @PostMapping("/customer/distributable/do")
    @ControllerMethod("保有客主动分配")
    public Message<Void> distributableDo(@CurrentUser LoginAuthBean currentUser, @RequestBody @Valid DistributableDTO dto) {
        List<Long> longList = leaveNeedDoBizService.activeAllocation(currentUser, dto);
        custAfterDistProducer.send(Boolean.FALSE, longList);
        return success();
    }
}
