package cn.fw.valhalla.controller.task

import cn.fw.valhalla.service.bus.follow.FollowBizService
import kotlinx.coroutines.CoroutineScope
import kotlinx.coroutines.Dispatchers
import kotlinx.coroutines.delay
import kotlinx.coroutines.launch
import org.slf4j.LoggerFactory
import org.springframework.beans.factory.annotation.Value
import org.springframework.boot.autoconfigure.condition.ConditionalOnProperty
import org.springframework.data.redis.core.StringRedisTemplate
import org.springframework.scheduling.annotation.Scheduled
import org.springframework.stereotype.Component

/**
 * 车辆进站处理task
 *
 * @author : kurisu
 * @version : 1.0
 * @desc : 车辆进站处理task
 * @date : 2024-03-01 09:51
 */
@Component
@ConditionalOnProperty(prefix = "task", name = ["switch"], havingValue = "on")
class InShopTask(
    private val followBizService: FollowBizService,
    private val stringRedisTemplate: StringRedisTemplate,
    @Value("\${spring.cache.custom.global-prefix}:need-stop-follow")
    private val needStopFollowKey: String
) {
    private val logger = LoggerFactory.getLogger(javaClass)


    /**
     * 停止线索推送推送待办
     */
    @Scheduled(initialDelay = 1000 * 10, fixedRate = 1000 * 5)
    fun stopClueTodo() {
        val boundSetOps = stringRedisTemplate.boundSetOps(needStopFollowKey)
        var idStr = boundSetOps.pop()
        var idx = 0
        val idList = mutableListOf<Long>()

        while (idStr != null && idx++ < (2 shl 6)) {
            idList.add(idStr.toLong())
            idStr = boundSetOps.pop()
        }
        if (idList.isEmpty()) {
            return
        }
        CoroutineScope(Dispatchers.IO).launch {
            idList.forEach {
                launch {
                    try {
                        followBizService.stopClueTodoForInShop(it)
                    } catch (e: Exception) {
                        logger.error("进站停止线索待办失败, id: $it", e)
                        delay(1500L)
                        boundSetOps.add(it.toString())
                    }
                }
            }
        }
    }
}