package cn.fw.valhalla.component.producer;

import cn.fw.valhalla.sdk.result.CustomerInfoDto;
import lombok.AccessLevel;
import lombok.Getter;
import lombok.extern.slf4j.Slf4j;
import org.apache.rocketmq.spring.core.RocketMQTemplate;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.beans.factory.annotation.Value;
import org.springframework.data.redis.core.SetOperations;
import org.springframework.data.redis.core.StringRedisTemplate;
import org.springframework.stereotype.Component;
import org.springframework.web.bind.annotation.RequestMapping;

import java.util.List;

/**
 * @author kurisu
 */
@Slf4j
@Component
public class CustomerChangeProducer {
    @Autowired
    private RocketMQTemplate rocketMQTemplate;
    @Autowired
    private StringRedisTemplate redisTemplate;
    @Value("${spring.cache.custom.global-prefix}:customer:change:prepare")
    @Getter(AccessLevel.PRIVATE)
    private String customerChangeKey;

    @RequestMapping(value = "send")
    public void send(CustomerInfoDto customerInfoDto) {
        try {
            log.info("售后保有客信息变更mq: body:[{}]", customerInfoDto);
            rocketMQTemplate.syncSend(CustomerInfoDto.TOPIC + ":*", customerInfoDto);
        } catch (Exception e) {
            e.printStackTrace();
        }
    }


    public void prepareSendCustomerChangeMq(Long customerId) {
        final String key = getCustomerChangeKey();
        SetOperations<String, String> opsForSet = redisTemplate.opsForSet();
        opsForSet.add(key, customerId.toString());
    }

    public void prepareSendCustomerChangeMq(List<Long> customerIdList) {
        final String key = getCustomerChangeKey();
        SetOperations<String, String> opsForSet = redisTemplate.opsForSet();
        String[] array = customerIdList.stream().map(String::valueOf).toArray(String[]::new);
        opsForSet.add(key, array);
    }
}