package cn.fw.valhalla.service.data.impl;

import cn.fw.valhalla.dao.mapper.CustomerReachLogMapper;
import cn.fw.valhalla.domain.db.customer.CustomerReachLog;
import cn.fw.valhalla.domain.enums.ReachInvalidReasonEnum;
import cn.fw.valhalla.service.data.CustomerReachLogService;
import com.baomidou.mybatisplus.core.toolkit.Wrappers;
import com.baomidou.mybatisplus.extension.service.impl.ServiceImpl;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Transactional;
import org.springframework.util.CollectionUtils;

import java.time.YearMonth;
import java.util.List;

/**
 * @author : kurisu
 * @className : CustomerReachLogServiceImpl
 * @description : 进站记录
 * @date: 2020-11-11 17:46
 */
@Service
public class CustomerReachLogServiceImpl extends ServiceImpl<CustomerReachLogMapper, CustomerReachLog> implements CustomerReachLogService {
    /**
     * 撤销排除统计
     *
     * @param customerId
     * @param shopId
     * @param adviserId
     * @param month
     */
    @Override
    @Transactional(rollbackFor = Exception.class)
    public void undoExclusion(final Long customerId, final Long shopId, final Long adviserId, final YearMonth month) {
        List<CustomerReachLog> list = this.list(Wrappers.<CustomerReachLog>lambdaUpdate()
                .eq(CustomerReachLog::getCustomerId, customerId)
                .eq(CustomerReachLog::getAdviserId, adviserId)
                .eq(CustomerReachLog::getShopId, shopId)
                .eq(CustomerReachLog::getReason, ReachInvalidReasonEnum.NATURAL)
                .eq(CustomerReachLog::getExcludeStatistics, Boolean.TRUE)
                .eq(CustomerReachLog::getDataMonth, month));
        if (CollectionUtils.isEmpty(list)) {
            return;
        }
        for (CustomerReachLog reachLog : list) {
            reachLog.setExcludeStatistics(Boolean.FALSE);
        }
        updateBatchById(list);
    }
}
